/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Original Code is DigitalSesame
 * Portions created by DigitalSesame are Copyright (C) 1997-2000 DigitalSesame
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: XHTMLDomFactory.java,v 1.2 2003/06/11 13:20:59 jkjome Exp $
 */

package org.enhydra.xml.xhtml;

import org.w3c.dom.*;
import org.enhydra.xml.xmlc.XMLCError;
import org.enhydra.xml.xmlc.dom.XMLCDomFactory;
import org.enhydra.xml.xmlc.dom.HTMLDomFactoryMethods;
import org.enhydra.xml.xmlc.dom.xerces.XercesDomFactory;
import org.enhydra.xml.xhtml.dom.xerces.XHTMLDOMImplementationImpl;
import org.enhydra.xml.xhtml.dom.XHTMLDocument;

import org.enhydra.apache.xerces.dom.DocumentTypeImpl;

/**
 * XMLC DOM factory for XHTML.
 * @see XMLCDomFactory
 */
public class XHTMLDomFactory extends XercesDomFactory implements XMLCDomFactory {
    private static final String IMPL_CLASS_SUFFIX = "Impl";
    private static final String XHTML_IMPLEMENTATION_DOT = "org.enhydra.xml.xhtml.dom.xerces.";

    private static final String XHTML_INTERFACE_DOT = "org.enhydra.xml.xhtml.dom.";

    private java.util.HashMap ClassNameMap = new java.util.HashMap();

    /*
     * Special-case element names to interface name that can't be
     * mapped just by changing the package.
     */
    private static final String[][] CLASS_INTERFACE_MAP = {
        /*
        ** Currently empty, but for example...
        **
        {"org.enhydra.apache.xerces.dom.ElementNSImpl", "org.w3c.dom.Element"},
        {"org.enhydra.apache.xerces.dom.AttrNSImpl", "org.w3c.dom.Attr"},
        */
    };

    /**
     * @see XMLCDomFactory#createDocumentType
     */
    public DocumentType createDocumentType(String qualifiedName,
                                           String publicId,
                                           String systemId,
                                           String internalSubset) {
        DOMImplementation domImpl = XHTMLDOMImplementationImpl.getHTMLDOMImplementation();
        DocumentTypeImpl docType =
            (DocumentTypeImpl)domImpl.createDocumentType(qualifiedName, publicId, systemId);
        docType.setInternalSubset(internalSubset);
        return docType;
    }

    /**
     * @see XMLCDomFactory#createDocument
     */
    public Document createDocument(String namespaceURI,
                                   String qualifiedName,
                                   DocumentType docType) {
        if (qualifiedName == null) {
            throw new XMLCError("qualifiedName is null for XHTML document;"
                                + " maybe trying to use a HTML parser on XHTML");
        }

	DOMImplementation domImpl = XHTMLDOMImplementationImpl.getHTMLDOMImplementation();
	return domImpl.createDocument(namespaceURI, qualifiedName, docType);
    }

    /**
     * The mime type for XHTML is a matter of debate.  There are arguments
     * for text/xml, text/xhtml, text/html, application/xml,
     * or application/xhtml+xml.  This returns application/xhtml+xml.
     * @see XMLCDomFactory#getMIMEType
     * @see HTMLDomFactory#getMIMEType
     */
    public String getMIMEType() {
        //return "text/xml";

        //more specific xhtml mime-type
        return "application/xhtml+xml";
    }

    /**
     * @see XMLCDomFactory#getBaseClassName
     */
    public String getBaseClassName() {
        return HTMLDomFactoryMethods.getBaseClassName();
    }

    /**
     * @see XMLCDomFactory#getInterfaceNames
     */
    public String[] getInterfaceNames() {
        return HTMLDomFactoryMethods.getInterfaceNames();
    }

    /**
     * @see XMLCDomFactory#nodeClassToInterface
     */
    public String nodeClassToInterface(Node node) {
        String className = node.getClass().getName();

        // Search explict class name mappings.
        for (int idx = 0; idx < CLASS_INTERFACE_MAP.length; idx++) {
            if (className.equals(CLASS_INTERFACE_MAP[idx][0])) {
                return CLASS_INTERFACE_MAP[idx][1];
            }
        }

	if (className.startsWith(XHTML_IMPLEMENTATION_DOT)) {
	    int suffixIdx = className.lastIndexOf(IMPL_CLASS_SUFFIX);
	    if (suffixIdx < 0) {
		throw new XMLCError("Class \"" + className
				    + "\" does not have suffix \"" + IMPL_CLASS_SUFFIX
				    + "\" (maybe be mismatch between XMLC code DOM implementation)");
	    }
            return XHTML_INTERFACE_DOT +
                className.substring(XHTML_IMPLEMENTATION_DOT.length(), suffixIdx);

	} else {
            // Unable to find or translate the node implementation class to a suitable
            // iterface, so throw an exception.
            throw new XMLCError("Can't determine DOM interface for node of class \""
                                + className
                                + "\" (maybe be mismatch between XMLCDomFactory DOM implementation)");

        }
    }

    /**
     * @see XMLCDomFactory#getElementClassNames
     */
    public String[] getElementClassNames(Element element) {
        return HTMLDomFactoryMethods.getElementClassNames(element);
    }

    /**
     * @see XMLCDomFactory#isURLAttribute
     */
    public boolean isURLAttribute(Element element,
                                  String attrName) {
        return HTMLDomFactoryMethods.isURLAttribute(element, attrName);
    }
}
