/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: JavaVariable.java,v 1.1.1.1 2003/03/10 16:36:19 taweili Exp $
 */

package org.enhydra.xml.xmlc.codegen;

import java.io.PrintWriter;

/**
 * Base class for classes that describe variables.
 * Objects of this class are immutable.
 */
abstract public class JavaVariable {
    /** Attributes of the variable */
    protected String fName;
    protected String fType;
    protected int fModifiers;
    protected String[] fDoc;

    /**
     * Constructor with doc array.
     * @param name The variable name.
     * @param type The variable type.
     * @param modifiers The variable modifier bit set.
     * @param doc The variable documentation, where each row is a line of the
     *  document.  If null, no documention is created.
     * @see JavaModifiers
     */
    public JavaVariable(String name,
                        String type,
                        int modifiers,
                        String[] doc) {
        fName = name;
        fType = type;
        fModifiers = modifiers;
        if ((doc != null) && (doc.length > 0)) {
            fDoc = (String[])doc.clone();
        }
    }

    /**
     * Constructor with doc string.
     * @param name The variable name.
     * @param type The variable type.
     * @param modifiers The variable modifier bit set.
     * @param doc The variable documentation.
     *  If null, no documention is created.
     * @see JavaModifiers
     */
    public JavaVariable(String name,
                        String type,
                        int modifiers,
                        String doc) {
        this(name, type, modifiers,
             (doc == null) ? null : new String[] {doc});
    }

    /**
     * Get the name.
     */
    public String getName() {
        return fName;
    }

    /**
     * Get the type.
     */
    public String getType() {
        return fType;
    }

    /**
     * Get the modifiers.
     */
    public int getModifiers() {
        return fModifiers;
    }

    /**
     * Get the documentation, or null if there is none.
     */
    protected String[] getDoc() {
        return fDoc;
    }

    /**
     * Output the variable definition without a `;' terminator.
     */
    public void printDefinition(PrintWriter out) {
        out.print(JavaModifiers.toDecl(fModifiers));
        out.print(fType);
        out.print(' ');
        out.print(fName);
    }
}
