/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DOMEditCmdOptions.java,v 1.1.1.1 2003/03/10 16:36:20 taweili Exp $
 */

package org.enhydra.xml.xmlc.commands.xmlc;

import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.XMLCError;
import org.enhydra.xml.io.ErrorReporter;
import org.enhydra.xml.xmlc.metadata.*;
import org.enhydra.xml.xmlc.commands.options.*;

/**
 * Command line options parsing DOM editing options.
 */
class DOMEditCmdOptions extends BaseCmdOptions {
    /**
     * Class for -urlmapping option.
     */
    private class UrlMappingOption extends Option {
        /**
         * Constructor.
         */
        public UrlMappingOption() {
            super("-urlmapping", 2, true,
                  "orgURL newURL - Specify a URL to modify and its new value");
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args,
                             ErrorReporter errorReporter,
                             Object clientData) throws XMLCException {
            URLMapping urlMapping
                = ((MetaData)clientData).getDOMEdits().addURLMapping();

            urlMapping.setUrl(args[0]);
            urlMapping.setNewUrl(args[1]);
        }
    }

    /**
     * Class for -urlsetting option.
     */
    private class UrlSettingOption extends Option {
        /**
         * Constructor.
         */
        public UrlSettingOption() {
            super("-urlsetting", 2, true,
                  "id newURL - Specify a URL to set given a tag id");
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args,
                             ErrorReporter errorReporter,
                             Object clientData) throws XMLCException {
            URLMapping urlMapping
                = ((MetaData)clientData).getDOMEdits().addURLMapping();

            urlMapping.addElementId(args[0]);
            urlMapping.setNewUrl(args[1]);
        }
    }

    /**
     * Class for -urlregexpmapping option.
     */
    private class UrlRegExpMappingOption extends Option {
        /**
         * Constructor.
         */
        public UrlRegExpMappingOption () {
            super("-urlregexpmapping", 2, true,
                  "regexp replace - Specify a regular expression to match URLs and a replacement pattern");
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args,
                             ErrorReporter errorReporter,
                             Object clientData) throws XMLCException {
            URLRegExpMapping urlRegExpMapping
                = ((MetaData)clientData).getDOMEdits().addURLRegExpMapping();
            urlRegExpMapping.setRegexp(args[0]);
            urlRegExpMapping.setSubst(args[1]);
        }
    }

    /**
     * Class for -delete-class option.
     */
    private class DeleteClassOption extends Option {
        /**
         * Constructor.
         */
        public DeleteClassOption() {
            super("-delete-class", 1, true,
                  "classname - Delete all elements are in the specified class (element class attribute, not Java class)");
        }

        /**
         * Parse an instance of the option.
         */
        protected void parse(String[] args,
                             ErrorReporter errorReporter,
                             Object clientData) throws XMLCException {
            DeleteElement deleteElement
                = ((MetaData)clientData).getDOMEdits().addDeleteElement();
            deleteElement.addElementClass(args[0]);
        }
    }

    /**
     * Constructor.  Add options to option set.
     */
    public DOMEditCmdOptions(OptionSet optionSet) {
        super(optionSet);
        optionSet.addOption(new UrlMappingOption());
        optionSet.addOption(new UrlSettingOption());
	optionSet.addOption(new UrlRegExpMappingOption());
	optionSet.addOption(new DeleteClassOption());
    }
}
