/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: GenerateType.java,v 1.1.1.1 2003/03/10 16:36:23 taweili Exp $
 */

package org.enhydra.xml.xmlc.metadata;

/**
 * Enumerated type indicating the type of class and/or interface XMLC
 * will generate.
 */
public class GenerateType extends EnumeratedType {
    /**
     * Constant indicating a standalone class should be generated.
     */
    public static final GenerateType CLASS = new GenerateType("class");

    /**
     * Constant indicating an interface should be generated.
     */
    public static final GenerateType INTERFACE = new GenerateType("interface");

    /**
     * Constant indicating an implementation should be generated.
     */
    public static final GenerateType IMPLEMENTATION = new GenerateType("implementation");

    /**
     * Constant indicating an interface and implementation should be generated.
     */
    public static final GenerateType BOTH = new GenerateType("both");

    /**
     * Internal constructor.  Only a fixed set of types are allowed.
     */
    private GenerateType(String name) {
        super(name);
    }

    /**
     * Does this type specify a class being generated?
     */
    public boolean generateClass() {
        return (this == CLASS) || (this == IMPLEMENTATION) || (this == BOTH);
    }

    /**
     * Does this type specify an interface being generated?
     */
    public boolean generateInterface() {
        return (this == INTERFACE) || (this == BOTH);
    }

    /**
     * Get the object for the specified type.  If desiredType is null,
     * returns null.
     */
    public static GenerateType getType(String desiredType) {
        if (desiredType == null) {
            return null;
        } else if (CLASS.fName.equals(desiredType)) {
            return CLASS;
        } else if (INTERFACE.fName.equals(desiredType)) {
            return INTERFACE;
        } else if (IMPLEMENTATION.fName.equals(desiredType)) {
            return IMPLEMENTATION;
        } else if (BOTH.fName.equals(desiredType)) {
            return BOTH;
        } else {
            throw new IllegalArgumentException("Invalid GenerateType: \""
                                               + desiredType
                                               + "\", expected one of \""
                                               + CLASS
                                               + "\", \", "
                                               + INTERFACE
                                               + "\", \""
                                               + IMPLEMENTATION
                                               + "\", \""
                                               + BOTH
                                               + "\"");
        }
    }
}
