/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BasicHTMLTests.java,v 1.1.1.1 2003/03/10 16:36:26 taweili Exp $
 */

package org.enhydra.xml.lazydom;
import java.lang.reflect.Method;
import org.enhydra.xml.xmlc.driver.XmlcTestParams;
import org.enhydra.xml.driver.TestException;
import org.enhydra.xml.driver.TestDomOps;
import org.enhydra.xml.lazydom.html.LazyHTMLDocument;
import org.enhydra.xml.io.DOMParser;
import org.enhydra.xml.dom.DOMInfo;
import java.lang.reflect.Method;
import java.io.File;
import junit.framework.Test;

/**
 * Basic tests of HTML Lazy DOM. XMLC coverts a lot of this too.
 */
public class BasicHTMLTests extends LazyDOMTestCaseBase {
    /** Input files */
    private static final String FRAME_TREE_HTML_DIR = "../xmlc/html";
    private static final String FRAME_TREE_HTML = "FrameTree.html";

    /** DOM builder and parser */
    private LazyDOMBuilder fBuilder
        = new LazyDOMBuilder(getMsgWriter(),
                             org.enhydra.xml.xmlc.dom.lazydom.LazyHTMLDomFactory.class);

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(BasicHTMLTests.class, null);
    }
    
    /** Constructor */
    public BasicHTMLTests(Method method) {
        super(method);
    }

    /** Create an unexpanded document from FrameTree.html */
    private LazyHTMLDocument parseFrameTree() {
        File frameTree = getOtherInputFile(FRAME_TREE_HTML_DIR, 
                                           FRAME_TREE_HTML);
        return (LazyHTMLDocument)fBuilder.parseUnexpanded(frameTree);
    }

    /**
     * Test 1: Simple test on FrameTree.html
     */
    public void test1() {
        LazyHTMLDocument lazyDoc = parseFrameTree();

        printDiffDom("Unexpanded Lazy DOM", lazyDoc,
                     "unexpanded.dom");
        TestDomOps.expandTree(lazyDoc);
        printDiffDom("Expanded Lazy DOM", lazyDoc,
                     "expanded.dom");
    }
}
