/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestCases.java,v 1.1.1.1 2003/03/10 16:36:27 taweili Exp $
 */

package org.enhydra.xml.oasis;

import java.util.ArrayList;
import java.io.PrintWriter;
import java.io.File;
import org.xml.sax.Attributes;

/**
 * Definition of a set of test cases.
 */
public class TestCases {
    /** Directory containing tests */
    private File fTestDir;

    /** Test profile */
    private String fProfile;

    /** Natual-language description of the test */
    private String fDescription = "";

    /** Parent TestCases */
    private TestCases fParent;

    /** Contained tests */
    private ArrayList fTests = new ArrayList();

    /** Contained test cases */
    private ArrayList fTestCases = new ArrayList();

    /** Is this test disabled? */
    private boolean fDisabled;

    /** Reasons for the test being disabled */
    private String fDisabledReason = "";

    /**
     * Constructor.
     */
    protected TestCases(TestCases parent,
                        File testDir,
                        Attributes attrs) {
        fParent = parent;
        fTestDir = testDir;
        fProfile = TestSuiteParser.getAttr(attrs, "PROFILE");
    }

    /**
     * Create a new TestCases.
     */
    TestCases createTestCases(Attributes attrs) {
        TestCases testCases = new TestCases(this, fTestDir, attrs);
        fTestCases.add(testCases);
        return testCases;
    }

    /**
     * Append characters to the description.
     */
    void appendDescription(String desc) {
        fDescription += desc;
    }

    /**
     * Append characters to the disabled reason.
     */
    void appendDisableReason(String reason) {
        fDisabledReason += reason;
    }

    /**
     * Flag as disabled.
     */
    void setDisabled() {
        fDisabled = true;
    }

    /**
     * Create a new Test.
     */
    Test createTest(Attributes attrs) {
        Test test = new Test(this, fTestDir, attrs);
        fTests.add(test);
        return test;
    }

    /**
     * Get the parent, or null if none.
     */
    public TestCases getParent() {
        return fParent;
    }

    /**
     * Get the test directory.
     */
    public File getTestDir() {
        return fTestDir;
    } 

    /**
     * Get the profile.
     */
    public String getProfile() {
        return fProfile;
    }

    /**
     * Get the natural-language description of the test.
     */
    public String getDescription() {
        return fDescription;
    }

    /**
     * Is the test disabled.
     */
    public boolean isDisabled() {
        return fDisabled;
    }

    /**
     * Get the reason the test was disabled.
     */
    public String getDisabledReason() {
        return fDisabledReason;
    }

    /**
     * Get the contained tests.
     */
    public Test[] getTests() {
        return (Test[])fTests.toArray(new Test[fTests.size()]);
    }

    /**
     * Get the contained test cases.
     */
    public TestCases[] getTestCases() {
        return (TestCases[])fTestCases.toArray(new TestCases[fTestCases.size()]);
    }

    /**
     * Print the contents.
     */
    public void print(PrintWriter writer,
                      int indent) {
        Print.printlnIndented(writer, indent, "TestCases: "
                              + getProfile());

        Test[] tests = getTests();
        for (int idx = 0; idx < tests.length; idx++) {
            tests[idx].print(writer, indent+1);
        }
        TestCases[] testCases = getTestCases();
        for (int idx = 0; idx < testCases.length; idx++) {
            testCases[idx].print(writer, indent);
        }
    }
}
