/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestSuite.java,v 1.1.1.1 2003/03/10 16:36:27 taweili Exp $
 */

package org.enhydra.xml.oasis;

import java.util.ArrayList;
import java.util.HashMap;
import java.io.PrintWriter;
import java.io.File;
import java.io.IOException;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import org.enhydra.xml.io.ErrorReporter;
import org.enhydra.xml.driver.TestException;

/**
 * Class that holds a definition of the OASIS XML Conformance test suite,
 * parsed from the XML definitions (testcases.dtd).
 */
public final class TestSuite extends TestCases {
    /**
     * Table of test cases by profile.
     */
    private HashMap fTestCasesByProfile = new HashMap();

    /**
     * Constructor.
     */
    protected TestSuite(File testDir,
                        Attributes attrs) {
        super(null, testDir, attrs);
    }

    /**
     * Record a set of test cases in the directories.
     */
    void recordTestCases(TestCases testCases) {
        String profile = testCases.getProfile();
        if (profile != null) {
            if (fTestCasesByProfile.containsKey(profile)) {
                throw new TestException("Dupliate TestCases profile: \""
                                           + profile + "\"");
            }
            fTestCasesByProfile.put(profile, testCases);
        }
    }

    /**
     * Print the contents.
     */
    public void print(PrintWriter writer,
                      int indent) {
        Print.printlnIndented(writer, indent, "TestSuite: " + getProfile());
        super.print(writer, indent+1);
    }

    /**
     * Factory method, parses the test conf document.
     */
    public static TestSuite parse(String testConf,
                                  ErrorReporter errorReporter)
        throws SAXException, IOException {

        File testDir = new File(testConf).getParentFile();
        if (testDir == null) {
            testDir = new File(".");
        }
        TestSuiteParser parser = new TestSuiteParser(testDir, errorReporter);
        TestSuite testSuite = parser.parse(new InputSource(testConf));
        if (errorReporter.getErrorCnt() > 0) {
            throw new SAXException("Errors parsing \"" + testConf + "\"");
        }
        return testSuite;
    }

    /**
     * Factory method, parses the test conf document with errors to stderr.
     */
    public static TestSuite parse(String testConf)
        throws SAXException, IOException {
        return parse(testConf, new ErrorReporter(System.err));
    }

    /**
     * Test main.
     */
    public static void main(String[] args) throws SAXException, IOException {
        TestSuite testSuite = parse("conf/xmlconf.xml");
        testSuite.print(new PrintWriter(System.out, true), 0);
    }
}
