/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: IOTests.java,v 1.1.1.1 2003/03/10 16:36:28 taweili Exp $
 */

package org.enhydra.xml.xmlc.html;
import org.enhydra.xml.xmlc.driver.XmlcTestParams;
import org.enhydra.xml.xmlc.driver.ExecXmlc;
import org.enhydra.xml.driver.TestError;
import java.lang.reflect.Method;
import junit.framework.Test;
import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;

/**
 * Tests of input/output related functionality.
 */
public class IOTests extends HtmlTestCaseBase {
    /** file: url of testpage1 */
    private String fTestPage1Url
        = "file:" + getInputFile("testPage1.html").getPath();

    /** Factory method to create suite of these tests */
    public static Test suite() {
        return createSuite(IOTests.class, null);
    }
    
    /** Constructor */
    public IOTests(Method method) {
        super(method);
    }

    /** Create a new test object */
    private HtmlBasicTest setupTest(File inputDoc) {
        return new HtmlBasicTest(this, inputDoc);
    }

    /** Create a new test object from a URL */
    private HtmlBasicTest setupTest(String inputDocURL) {
        try {
            URL url = new URL(inputDocURL);
            return setupTest(new File(url.getFile()));
        } catch (MalformedURLException except) {
            throw new TestError(except);
        }
    }

    /** Create a new test object for testPage1 */
    private HtmlBasicTest setupTest() {
        return setupTest(fTestPage1Url);
    }

    /*
     * Test 1: explicit file URI
     */
    private void disabled_test1() {
        // FIXME: Disabled, don't know how to test with input dir mapping
        HtmlBasicTest test = setupTest();
        test.basicTest();
    }

    /*
     * Test 2: ../xxx relative file 
     */
    private void disabled_test2() {
        // FIXME: Disabled, don't know how to test with input dir mapping
        // HtmlBasicTest test = setupTest("../html/input/testPage1.html");
        // test.basicTest();
    }

    /*
     * Test 3: ../../xxx relative file.
     */
    private void disabled_test3() {
        // FIXME: Disabled, don't know how to test with input dir mapping
        // HtmlBasicTest test = setupTest("../../xmlc/html/input/testPage1.html");
        // test.basicTest();
    }

    /*
     * Test 4: specifying public id
     */
    public void test4() {
        HtmlBasicTest test = setupTest();
        test.getOutputOptions().setPublicId("-publicid -//W3C//DTD@HTML@4.0//EN");
        test.basicTest();
    }

    /*
     * Test 5: specifying public and system id
     */
    public void test5() {
        HtmlBasicTest test = setupTest();
        test.getOutputOptions().setPublicId("-//W3C//DTD@HTML@4.0//EN");
        test.getOutputOptions().setSystemId("http://www.w3.org/TR/html4/strict.dtd");
        test.basicTest();
    }

    /*
     * Test 6: specifying system id
     */
    public void test6() {
        HtmlBasicTest test = setupTest();
        test.getOutputOptions().setSystemId("http://www.w3.org/TR/html4/strict.dtd");
        test.basicTest();
    }
}
