/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestInfoUpdate.java,v 1.1.1.1 2003/03/10 16:37:12 taweili Exp $
 */

package org.enhydra.xml.xmlc.servlet.src;

import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.xmlc.XMLCFactory;
import org.enhydra.xml.xmlc.reloading.XMLCReloadingFactory;
import org.enhydra.xml.xmlc.servlet.XMLCContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpUtils;
import javax.servlet.http.HttpSession;

/**
 * Store various statistics into a document implementing TestInfo
 */
class TestInfoUpdate {
    /**
     * Message
     */
    private static final String NA = "n.a.";
    
    /**
     * Name of counter in session.
     */
    private static final String SESSION_COUNTER_NAME = "sessionCounter";

    /**
     * Get reloading count from a factory.
     */
    private int getReloadCount(XMLCFactory factory) {
        if (factory instanceof XMLCReloadingFactory) {
            return ((XMLCReloadingFactory)factory).getReloadCount();
        } else {
            return 0;
        }
    }

    /**
     * Get recompilation count from a factory
     */
    private int getRecompileCount(XMLCFactory factory) {
        if (factory instanceof XMLCReloadingFactory) {
            return ((XMLCReloadingFactory)factory).getRecompileCount();
        } else {
            return 0;
        }
    }

    /**
     * Store test information into a page.
     */
    public void setTestInfo(TestInfo doc,
                            XMLCContext context,
                            HttpServletRequest request) {
        XMLCFactory factory = context.getXMLCFactory();
                
        doc.setTextRequestURI(request.getRequestURI());
        doc.setTextRequestURL(HttpUtils.getRequestURL(request).toString());
        doc.setTextSessionValid(new Boolean(request.isRequestedSessionIdValid()).toString());
        doc.setTextSessionFromCookie(new Boolean(request.isRequestedSessionIdFromCookie()).toString());
        doc.setTextSessionFromURL(new Boolean(request.isRequestedSessionIdFromURL()) .toString());
        doc.setTextUrlEncoding(context.getSessionURLEncoding().toString());

        if (request.isRequestedSessionIdValid()) {
            HttpSession session = request.getSession(false);
            Integer sessionCounter = (Integer)session.getAttribute(SESSION_COUNTER_NAME);

            doc.setTextSessionId(session.getId());
            doc.setTextSessionCounter((sessionCounter == null) ? "null" : sessionCounter.toString());

            if (sessionCounter == null) {
                sessionCounter = new Integer(1);
            } else {
                sessionCounter = new Integer(sessionCounter.intValue() + 1);
            }
            session.setAttribute(SESSION_COUNTER_NAME, sessionCounter);
        } else {
            doc.setTextSessionId(NA);
            doc.setTextSessionCounter(NA);
        }

        if (factory instanceof XMLCReloadingFactory) {
            doc.setTextReloadCount(Integer.toString(getReloadCount(factory)));
            doc.setTextRecompileCount(Integer.toString(getRecompileCount(factory)));
        } else {
            doc.setTextReloadCount(NA);
            doc.setTextRecompileCount(NA);
        }
    }


    /**
     * Store test information in the page, if it implements TestInfo.
     */
    public void setTestInfo(XMLObject doc,
                            XMLCContext context,
                            HttpServletRequest request) {
        if (doc instanceof TestInfo) {
            setTestInfo((TestInfo)doc, context, request);
        }
    }
}
